use std::sync::Arc;

use anyhow::Result;
use libpt::log::warn;
use minijinja::context;
use minijinja::Environment;
use serde;
use warp::Filter;

use crate::config::Config;
use crate::vault::VaultRef;

#[derive(Clone)]
pub struct Service<'tp> {
    vault: VaultRef,
    config: Config,
    env: Environment<'tp>,
}
impl<'tp> Service<'tp> {
    fn new(vault: VaultRef, config: Config, env: Environment<'tp>) -> Arc<Self> {
        Self { vault, config, env }.into()
    }
}

pub async fn serve(vault: VaultRef, config: Config) -> Result<()> {
    let mut env = Environment::new();
    env.add_template("index", include_str!("../../data/www/admin.html"))?;

    let service = Service::new(vault, config, env);
    let service2 = service.clone();

    let routes = warp::path::end()
        .map(move || service2.clone())
        .and_then(overview);

    warp::serve(routes)
        .run(service.config.addr_admin.unwrap())
        .await;

    warn!("exited the admin interface");
    Ok(())
}

async fn overview(serv: Arc<Service<'_>>) -> Result<Box<dyn warp::Reply>, warp::Rejection> {
    let r = serv
        .env
        .get_template("index")
        .unwrap()
        .render(context!(TITLE => "Wooly-Vault", AUTHOR => env!("CARGO_PKG_AUTHORS")))
        .unwrap();

    Ok(Box::new(r))
}

async fn styles() -> Result<Box<dyn warp::Reply>, warp::Rejection> {
    let r = include_str!("../../data/www/styles.css").to_string();
    Ok(Box::new(r))
}
